/*
 * Copyright (c) 2024 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <string.h>
#include <stdint.h>
#include <hal/nrf_ccm.h>
#include <zephyr/kernel.h>
#include <zephyr/ztest.h>
#include "common.h"

#define CCM NRF_CCM00
#define ENABLE_DATA_PRINTOUT false

enum protocol_t { BLE_MODE = CCM_MODE_PROTOCOL_Ble, IEEE_MODE = CCM_MODE_PROTOCOL_Ieee802154 };

static void run_ccm(uint32_t key[], uint32_t nonce[], job_t *p_injob, job_t *p_outjob,
    unsigned maclen, uint8_t adatamask, bool expect_error, bool expect_macerror,
    bool decrypt, enum protocol_t mode_protocol)
{
  nrf_ccm_enable(CCM);

  nrf_ccm_event_clear(CCM, NRF_CCM_EVENT_END);
  nrf_ccm_event_clear(CCM, NRF_CCM_EVENT_ERROR);

  nrf_ccm_adatamask_set(CCM, adatamask);

  nrf_ccm_key_set(CCM, key);
  nrf_ccm_nonce_set(CCM, nonce);

  nrf_ccm_in_ptr_set(CCM, (nrf_vdma_job_t const *)p_injob);
  nrf_ccm_out_ptr_set(CCM, (nrf_vdma_job_t const *)p_outjob);

  if (ENABLE_DATA_PRINTOUT) {
    printk("Key in registers:\n");
    print_uint32_array(key, 4);
    printk("Nonce in registers:\n");
    print_uint32_array(nonce, 4);
    printk("Input job:\n");
    print_job(p_injob, true);
  }

  unsigned mac_config_val = maclen ? ((maclen - 2) / 2) << CCM_MODE_MACLEN_Pos : 0;

  CCM->MODE = mac_config_val | (mode_protocol << CCM_MODE_PROTOCOL_Pos);
  if (decrypt) {
    CCM->MODE |= (CCM_MODE_MODE_Decryption << CCM_MODE_MODE_Pos);
  } else {
    CCM->MODE |= (CCM_MODE_MODE_Encryption << CCM_MODE_MODE_Pos);
  }

  nrf_ccm_task_trigger(CCM, NRF_CCM_TASK_START);

  while (!nrf_ccm_event_check(CCM, NRF_CCM_EVENT_END) && !nrf_ccm_event_check(CCM, NRF_CCM_EVENT_ERROR))
  {
    k_busy_wait(1);
  }

  if (ENABLE_DATA_PRINTOUT) {
    printk("Processing done\n");
    printk("Output job:\n");
    print_job(p_outjob, true);
  }

  zassert_equal(CCM->EVENTS_ERROR, (uint32_t)expect_error, "CCM EVENTS_ERROR is %u but expected %u!",
      CCM->EVENTS_ERROR, expect_error);
  zassert_true(!decrypt || CCM->MACSTATUS == (uint32_t)!expect_macerror,
      "MACSTATUS %u is wrong, decrypt is %u expect_macerror is %u!\n", CCM->MACSTATUS, decrypt,
      expect_macerror);

  nrf_ccm_event_clear(CCM, NRF_CCM_EVENT_END);
  nrf_ccm_event_clear(CCM, NRF_CCM_EVENT_ERROR);

  nrf_ccm_disable(CCM);
}

static void print_and_verify_bytearray(uint8_t *p_vals, uint8_t *p_expected_vals, unsigned size)
{
  if (ENABLE_DATA_PRINTOUT) {
    printk("0x");
  }
  for (size_t i = 0; i < size; i++) {
    if (ENABLE_DATA_PRINTOUT) {
      printk("%02X", p_vals[i]);
    }
    zassert_equal(p_vals[i], p_expected_vals[i], "Byte %d (0x%02X) does not match the expected output (0x%02X)!",
        i, p_vals[i], p_expected_vals[i]);
  }
  printk(" matches the expected output!\n");
}

static void build_ccm_test_case_and_run(uint8_t K[], uint8_t N[], uint8_t *P, uint16_t P_size, uint16_t mlen,
    uint8_t *A, uint16_t alen, uint8_t *C, uint16_t out_data_len, unsigned mac_len,
    uint8_t adata_mask, bool expect_error, bool expect_macerror, bool decrypt,
    enum protocol_t mode_protocol, uint16_t *p_mlen_out, bool expect_out_data_len_mismatch)
{
  uint8_t key_reverse[16];
  uint8_t nonce_reverse[16];
  reverse_bytearray(K, key_reverse, 16);
  reverse_bytearray(N, nonce_reverse, 16);
  uint32_t *nonce = (uint32_t *)nonce_reverse;
  uint32_t *key = (uint32_t *)key_reverse;

  job_t ccm_injob[] = {
      {(uint8_t *)&alen, sizeof(alen), CCM_DMA_ATTR_ALEN},
      {(uint8_t *)&mlen, sizeof(mlen), CCM_DMA_ATTR_MLEN},
      {(uint8_t *)A, alen, CCM_DMA_ATTR_ADATA},
      {(uint8_t *)P, P_size, CCM_DMA_ATTR_MDATA},
      {0x0, 0x0} // Marking end of job list
  };

  uint16_t alen_out = 0xffff;
  uint16_t mlen_out = 0xffff;
  uint8_t adata_out[alen];
  memset(adata_out, 0xff, alen);
  const uint32_t PAINT = 0xCABACABA;
  uint8_t out_data[out_data_len + 2 * sizeof(uint32_t)];
  memset(out_data, 0xff, out_data_len + 2 * sizeof(uint32_t));
  uint32_t *p_inside_paint = (uint32_t *)&out_data[0];
  uint32_t *p_outside_paint = (uint32_t *)&out_data[sizeof(out_data) - sizeof(uint32_t)];
  *p_inside_paint = PAINT;
  *p_outside_paint = PAINT;
  memset(&out_data[sizeof(uint32_t)], 0xff, out_data_len);

  job_t ccm_outjob[] = {
      {(uint8_t *)&alen_out, sizeof(alen_out), CCM_DMA_ATTR_ALEN},
      {(uint8_t *)&mlen_out, sizeof(mlen_out), CCM_DMA_ATTR_MLEN},
      {(uint8_t *)adata_out, sizeof(adata_out), CCM_DMA_ATTR_ADATA},
      {(uint8_t *)&out_data[sizeof(uint32_t)], out_data_len, CCM_DMA_ATTR_MDATA},
      {0x0, 0x0} // Marking end of job list
  };

  run_ccm(key, nonce, ccm_injob, ccm_outjob, mac_len, adata_mask, expect_error, expect_macerror, decrypt,
      mode_protocol);
  zassert_equal(alen_out, sizeof(adata_out), "Header data length (0x%02X) does not match expected size (0x%02X)\n",
      adata_out, sizeof(adata_out));
  zassert_equal(*p_inside_paint, PAINT, "Bufer overflow\n");
  zassert_equal(*p_outside_paint, PAINT, "Bufer overflow\n");
  if (!expect_macerror && !expect_error) {
    zassert_true(expect_out_data_len_mismatch || (mlen_out == out_data_len),
        "Out data length (0x%02X) does not match expected size (0x%02X)\n", mlen_out, out_data_len);
    if (decrypt) {
      printk("Decrypted data: ");
    } else {
      printk("Encrypted data: ");
    }
    print_and_verify_bytearray(&out_data[sizeof(uint32_t)], C, out_data_len);
    printk("Header (Adata): ");
    print_and_verify_bytearray(adata_out, A, alen_out);
  }

  if (p_mlen_out != NULL) {
    *p_mlen_out = mlen_out;
  }
}

static const bool DECRYPT = true;
static const bool ENCRYPT = false;
static const bool EXPECT_MAC_ERROR = true;
static const bool EXPECT_ERROR = true;

ZTEST(nrf_ccm_tests, test_ccm_00_ble_spec_example_packet1)
{
  printk("Test data from BT Core specification 6.0, Volume 6, Part C, chapter 1.2, packet \"1.START_ENC_RSP1\"\n");

  //SK = 99AD1B5226A37E3E058E3B8E27C2C666
  uint8_t K[16] = {0x99, 0xad, 0x1b, 0x52, 0x26, 0xa3, 0x7e, 0x3e, 0x05, 0x8e, 0x3b, 0x8e, 0x27, 0xc2, 0xc6, 0x66};
  //IV = DEAFBABEBADCAB24
  //Packet counter = 0, Direction = 1 (central -> periph)
  uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
  uint8_t A[] = {0x0F}; //AAD = First header byte
  uint8_t P[] = {0x06};
  uint8_t C[] = {0x9f, 0xcd, 0xa7, 0xf4, 0x48};
  unsigned mac_len = 4;
  uint8_t adata_mask = 0xE3;
  build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
      false, ENCRYPT, BLE_MODE, NULL, false);
}

ZTEST(nrf_ccm_tests, test_ccm_00_ble_spec_example_packet2)
{
  printk("Test data from BT Core specification 6.0, Volume 6, Part C, chapter 1.2, packet \"2.START_ENC_RSP2\"\n");

  //SK = 99AD1B5226A37E3E058E3B8E27C2C666
  uint8_t K[16] = {0x99, 0xad, 0x1b, 0x52, 0x26, 0xa3, 0x7e, 0x3e, 0x05, 0x8e, 0x3b, 0x8e, 0x27, 0xc2, 0xc6, 0x66};
  //IV = DEAFBABEBADCAB24
  //Packet counter = 0, Direction = 0 (periph -> central)
  uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
  uint8_t A[] = {0x07}; //AAD = First header byte
  uint8_t P[] = {0x06};
  uint8_t C[] = {0xA3, 0x4C, 0x13, 0xA4, 0x15};

  unsigned mac_len = 4;
  uint8_t adata_mask = 0xE3;
  build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
      false, ENCRYPT, BLE_MODE, NULL, false);
}

ZTEST(nrf_ccm_tests, test_ccm_00_ble_spec_example_packet3)
{
  printk("Test data from BT Core specification 6.0, Volume 6, Part C, chapter 1.2, packet \"3. Data packet1\"\n");

  //SK = 99AD1B5226A37E3E058E3B8E27C2C666
  uint8_t K[16] = {0x99, 0xad, 0x1b, 0x52, 0x26, 0xa3, 0x7e, 0x3e, 0x05, 0x8e, 0x3b, 0x8e, 0x27, 0xc2, 0xc6, 0x66};
  //IV = DEAFBABEBADCAB24
  //Packet counter = 1, Direction = 1 (central -> periph):
  uint8_t N[16] = {0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
  uint8_t A[] = {0x0E}; //AAD = First header byte
  uint8_t P[] = {0x17, 0x00, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f, 0x70,
                 0x71, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x30};
  uint8_t C[] = {0x7A, 0x70, 0xD6, 0x64, 0x15, 0x22, 0x6D, 0xF2, 0x6B, 0x17, 0x83, 0x9A, 0x06, 0x04, 0x05, 0x59,
                 0x6B, 0xD6, 0x56, 0x4F, 0x79, 0x6B, 0x5B, 0x9C, 0xE6, 0xFF, 0x32, 0xF7, 0x5A, 0x6D, 0x33};

  unsigned mac_len = 4;
  uint8_t adata_mask = 0xE3;
  build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
      false, ENCRYPT, BLE_MODE, NULL, false);
}

ZTEST(nrf_ccm_tests, test_ccm_00_ble_spec_example_packet4)
{
  printk("Test data from BT Core specification 6.0, Volume 6, Part C, chapter 1.2, packet \"4. Data packet2\"\n");

  //SK = 99AD1B5226A37E3E058E3B8E27C2C666
  uint8_t K[16] = {0x99, 0xad, 0x1b, 0x52, 0x26, 0xa3, 0x7e, 0x3e, 0x05, 0x8e, 0x3b, 0x8e, 0x27, 0xc2, 0xc6, 0x66};
  //IV = DEAFBABEBADCAB24
  //Packet counter = 1, Direction = 0 (periph -> central)
  uint8_t N[16] = {0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
  uint8_t A[] = {0x06}; //AAD = First header byte
  uint8_t P[] = {0x17, 0x00, 0x37, 0x36, 0x35, 0x34, 0x33, 0x32, 0x31, 0x30, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46,
                 0x47, 0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f, 0x50, 0x51};
  uint8_t C[] = {0xF3, 0x88, 0x81, 0xE7, 0xBD, 0x94, 0xC9, 0xC3, 0x69, 0xB9, 0xA6, 0x68, 0x46, 0xDD, 0x47, 0x86,
                 0xAA, 0x8C, 0x39, 0xCE, 0x54, 0x0D, 0x0D, 0xAE, 0x3A, 0xDC, 0xDF, 0x89, 0xB9, 0x60, 0x88};
  unsigned mac_len = 4;
  uint8_t adata_mask = 0xE3;
  build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
      false, ENCRYPT, BLE_MODE, NULL, false);
}


ZTEST(nrf_ccm_tests, test_ccm_01_decr_1to255)
{
	printk("Test decrypting vector of sizes 1 - 255 with mdata length + 8\n");

	uint8_t K[16] = {0x99, 0xAD, 0x1B, 0x52, 0x26, 0xA3, 0x7E, 0x3E, 0x05, 0x8E, 0x3B, 0x8E, 0x27, 0xC2, 0xC6, 0x66};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
	uint8_t A[] = {0x0E};
	uint8_t P[255];
	static uint8_t EncryptedP[] = {
			0x6D, 0x71, 0xB7, 0x03, 0x74, 0x41, 0x0C, 0x9D, 0x0A, 0x74, 0xE2, 0xFD, 0x67, 0x67, 0x64, 0x26, 0x0A, 0xF6,
			0x76, 0x6F, 0x59, 0x4B, 0x7B, 0xBC, 0xC6, 0xDF, 0x18, 0x5B, 0x50, 0x2D, 0x3D, 0x5E, 0xD9, 0xAF, 0xF6, 0xB8,
			0xFE, 0x86, 0xC5, 0x23, 0x3A, 0xA8, 0x35, 0xE4, 0xE6, 0xA0, 0xB6, 0xC9, 0x48, 0xFF, 0xFF, 0xF5, 0xA0, 0xC0,
			0x19, 0x32, 0x71, 0x60, 0x61, 0x0C, 0x1B, 0xE5, 0x43, 0xCE, 0xB5, 0xBC, 0x9E, 0x3A, 0x3D, 0x85, 0xB7, 0xAA,
			0x65, 0x64, 0x2E, 0x38, 0x82, 0x27, 0x3D, 0xA4, 0x13, 0xFE, 0xE5, 0x89, 0x1F, 0x2F, 0x0E, 0x9B, 0x02, 0x0B,
			0x10, 0xCD, 0xA8, 0xCD, 0xBA, 0xA6, 0xD0, 0x40, 0x44, 0xD0, 0x75, 0x37, 0xBB, 0x06, 0x16, 0x54, 0x04, 0x7F,
			0x8A, 0x0C, 0x2E, 0x7E, 0xB8, 0x06, 0x06, 0x63, 0x3F, 0x4A, 0x08, 0xFF, 0x21, 0x28, 0x6C, 0x8F, 0x65, 0xB6,
			0x29, 0xE5, 0xFC, 0x58, 0x29, 0x7D, 0x55, 0xFE, 0xA8, 0x24, 0xDE, 0xE5, 0xB1, 0x8C, 0x9B, 0xAF, 0xD3, 0x78,
			0xCE, 0x87, 0x92, 0x07, 0x4F, 0x40, 0x75, 0xE6, 0xA7, 0x61, 0x09, 0x73, 0xBE, 0x0A, 0x29, 0x05, 0x3F, 0xF7,
			0xBD, 0xD8, 0x9C, 0xC7, 0xDA, 0x4B, 0xAC, 0xE9, 0xC2, 0xE4, 0xD7, 0x4C, 0x2C, 0x26, 0x8C, 0x00, 0xE4, 0xE6,
			0xBE, 0x05, 0x0E, 0xC2, 0xDA, 0x1E, 0xA7, 0xAE, 0xD7, 0x35, 0xA8, 0x26, 0xCD, 0xF8, 0x93, 0x93, 0x26, 0x0C,
			0xD5, 0x47, 0x9B, 0x7C, 0x35, 0x62, 0x7D, 0xFB, 0xCF, 0xAD, 0x78, 0x91, 0xA6, 0x0C, 0xC8, 0x5C, 0xBF, 0x05,
			0xBF, 0x7E, 0xE4, 0x0C, 0x4A, 0x9E, 0xC9, 0xD2, 0x9F, 0x75, 0x7A, 0x58, 0x84, 0x9B, 0xD4, 0x9A, 0x2B, 0x51,
			0xB2, 0x4A, 0x3A, 0x45, 0xEB, 0x58, 0x33, 0x86, 0x53, 0xC5, 0xBA, 0x63, 0x73, 0xB6, 0x89, 0x25, 0xD7};
	static uint8_t MAC_VALS[251][4] = {
			{0xD4, 0x6B, 0xFF, 0x83}, {0x65, 0x2C, 0xF0, 0xE7}, {0xA1, 0x46, 0xB3, 0x47}, {0x8D, 0x59, 0xCD, 0x00},
			{0xF0, 0x30, 0xEB, 0x37}, {0x7B, 0xA5, 0x1D, 0x70}, {0x6F, 0xEC, 0x4F, 0xC1}, {0x17, 0x20, 0x0F, 0x1E},
			{0x00, 0x35, 0x68, 0xA0}, {0x9F, 0x15, 0xFE, 0x4A}, {0xC4, 0xD9, 0x23, 0xCD}, {0xD3, 0x66, 0xB5, 0x62},
			{0x14, 0x9B, 0x81, 0x4B}, {0x2E, 0x41, 0xCA, 0x25}, {0xD6, 0x84, 0x29, 0x22}, {0x2F, 0xBB, 0xD5, 0xFF},
			{0x15, 0x30, 0xCB, 0xA1}, {0x04, 0xC8, 0x39, 0xAD}, {0x99, 0x7B, 0x0E, 0xF8}, {0x0C, 0x02, 0x46, 0x6D},
			{0x66, 0xF2, 0x2F, 0xD2}, {0xF0, 0x20, 0x90, 0x79}, {0xC6, 0x2B, 0x09, 0x82}, {0xC4, 0x36, 0x74, 0xF2},
			{0x5D, 0x05, 0x49, 0x80}, {0x17, 0xF9, 0x8B, 0x07}, {0x61, 0xC2, 0x72, 0x45}, {0x0B, 0x73, 0x2B, 0xCA},
			{0xCE, 0x7B, 0x7F, 0xEE}, {0xB0, 0x3E, 0xA3, 0x30}, {0x3D, 0x05, 0xCF, 0x32}, {0x54, 0xF6, 0x18, 0xCE},
			{0xDC, 0xB3, 0x78, 0xD2}, {0xD5, 0x13, 0x49, 0x91}, {0xDE, 0xAE, 0xA6, 0x1E}, {0xFB, 0x67, 0xF6, 0x3B},
			{0xEE, 0xDE, 0xA7, 0xF4}, {0x94, 0xD3, 0x43, 0x76}, {0x2A, 0xB8, 0x98, 0xD7}, {0x5F, 0x44, 0x18, 0x95},
			{0x33, 0x7F, 0xF5, 0x3C}, {0x78, 0x05, 0x05, 0xBB}, {0x59, 0x34, 0xCB, 0x2E}, {0x5E, 0x23, 0x6B, 0x2D},
			{0x8A, 0x4A, 0x9A, 0x85}, {0x15, 0x7B, 0x48, 0x67}, {0xC1, 0x82, 0x87, 0x8C}, {0x38, 0x8A, 0xB2, 0x14},
			{0xFD, 0xE7, 0x0C, 0x9C}, {0x04, 0x77, 0x29, 0xE3}, {0x4E, 0x0E, 0xE1, 0xA2}, {0x37, 0x4D, 0x06, 0x51},
			{0x78, 0x1B, 0x03, 0x50}, {0x9C, 0xDB, 0x02, 0x90}, {0x77, 0x97, 0xB6, 0xB7}, {0xEF, 0x0D, 0x96, 0xC4},
			{0x08, 0xD9, 0x9F, 0xA0}, {0xEA, 0x9C, 0x92, 0xAD}, {0x64, 0x4E, 0xBB, 0x18}, {0xD7, 0x60, 0xB2, 0xCB},
			{0x4D, 0x7D, 0xB7, 0x2A}, {0x50, 0x4A, 0x02, 0xBF}, {0x28, 0x1A, 0xCF, 0x0B}, {0x83, 0x11, 0x9C, 0x14},
			{0x25, 0x3C, 0x13, 0x01}, {0x4F, 0x63, 0x8F, 0xC5}, {0x06, 0xF0, 0xD8, 0xDA}, {0xD0, 0x5E, 0xA6, 0xAE},
			{0x9F, 0x8D, 0x77, 0x37}, {0xB8, 0xDA, 0x8C, 0x07}, {0xB6, 0x9F, 0x67, 0xC0}, {0x24, 0xC9, 0x82, 0x0C},
			{0x75, 0xB2, 0x9C, 0x4F}, {0x10, 0xB9, 0x07, 0x8F}, {0x5F, 0x9F, 0x75, 0x00}, {0x67, 0x67, 0xC7, 0x64},
			{0x20, 0xFE, 0x12, 0x28}, {0x22, 0x38, 0x58, 0xAC}, {0xEB, 0x02, 0x25, 0x69}, {0x4C, 0xF5, 0x6C, 0x13},
			{0xF4, 0x4F, 0xB1, 0xA1}, {0xA5, 0xD0, 0xE7, 0x08}, {0x1F, 0xD6, 0x30, 0x07}, {0x1E, 0xF2, 0xA3, 0xF3},
			{0xCD, 0xC9, 0xFA, 0x28}, {0x54, 0x9F, 0x42, 0xD0}, {0x81, 0xB8, 0xA6, 0x4E}, {0x0D, 0xBD, 0xB0, 0x0B},
			{0xCF, 0x7D, 0x7B, 0x30}, {0xC9, 0xB7, 0xD6, 0x42}, {0xA5, 0x09, 0x7B, 0x30}, {0x3B, 0x81, 0x66, 0xF1},
			{0x1B, 0x33, 0x69, 0x92}, {0xB6, 0x2D, 0xCF, 0xEB}, {0x6F, 0xC9, 0x61, 0xD0}, {0x52, 0x10, 0xB1, 0xB0},
			{0xF0, 0xE9, 0xD1, 0x8D}, {0xF2, 0x04, 0x3A, 0xBA}, {0xBB, 0xF4, 0xA6, 0x44}, {0x86, 0x1B, 0x50, 0x86},
			{0x52, 0x25, 0x70, 0x36}, {0x53, 0xFB, 0x00, 0x00}, {0x1A, 0x61, 0x7B, 0x51}, {0x57, 0xAA, 0xFA, 0x04},
			{0xE4, 0xFD, 0x03, 0x48}, {0x69, 0x58, 0x75, 0xD0}, {0x16, 0x9C, 0x23, 0x63}, {0xD1, 0x59, 0x3C, 0xC6},
			{0x22, 0x57, 0xAC, 0x97}, {0x5D, 0x60, 0x95, 0xB6}, {0x70, 0xEB, 0xF3, 0x7C}, {0xB8, 0x49, 0x2F, 0x87},
			{0x97, 0x23, 0x63, 0xB6}, {0x59, 0x56, 0xC5, 0x0F}, {0xDD, 0x06, 0x6E, 0x63}, {0x79, 0x44, 0xF1, 0x9A},
			{0x30, 0x8C, 0xF9, 0xAD}, {0x10, 0xFD, 0xDB, 0x1C}, {0xC9, 0xCC, 0x98, 0xA8}, {0x57, 0x4A, 0xF6, 0x6F},
			{0x70, 0x92, 0x44, 0x83}, {0x07, 0x56, 0x81, 0xEF}, {0xA6, 0x85, 0x29, 0x46}, {0x60, 0xF4, 0xD4, 0x2A},
			{0xFF, 0x2A, 0x6E, 0xFE}, {0xAE, 0x4F, 0xF7, 0x37}, {0xEE, 0xD1, 0x41, 0xE4}, {0xC1, 0xEA, 0xE3, 0x9B},
			{0x8D, 0x87, 0x9A, 0xF3}, {0x7E, 0xB2, 0xB4, 0x51}, {0x2B, 0xDA, 0xF8, 0x94}, {0xB7, 0x82, 0x9E, 0x0C},
			{0xFB, 0x85, 0xE0, 0x9F}, {0xE9, 0x60, 0x51, 0xB3}, {0xE3, 0x6D, 0x76, 0xD4}, {0x8A, 0xA2, 0x7A, 0x0C},
			{0xEF, 0x95, 0x07, 0x9E}, {0x8A, 0xD9, 0xDF, 0xE6}, {0x36, 0x17, 0xAD, 0x43}, {0x5E, 0xED, 0xDA, 0xF6},
			{0xB6, 0xB5, 0x84, 0x2F}, {0x39, 0x5F, 0x86, 0xF5}, {0x25, 0x88, 0x4C, 0x61}, {0xA2, 0xFC, 0xEE, 0x6D},
			{0x2E, 0xD6, 0xC5, 0xCF}, {0x3C, 0x08, 0xA1, 0xC8}, {0xAA, 0x06, 0x98, 0x52}, {0xB8, 0x4D, 0x43, 0x63},
			{0xAB, 0x1F, 0xE4, 0xA3}, {0xDE, 0x5F, 0x59, 0x2D}, {0x5C, 0xDA, 0x6C, 0x43}, {0x80, 0x58, 0xFC, 0xF2},
			{0xFB, 0x14, 0x9F, 0x3A}, {0xF5, 0x22, 0x08, 0x74}, {0x8D, 0x94, 0x7D, 0x2E}, {0xD1, 0x91, 0x83, 0xB0},
			{0xFE, 0x78, 0x7A, 0x02}, {0xB5, 0x7A, 0x4C, 0xF7}, {0xBA, 0x87, 0xF9, 0x56}, {0x9D, 0x37, 0xBA, 0xCE},
			{0x9E, 0x5C, 0x1E, 0xC3}, {0x91, 0x98, 0x18, 0x48}, {0xA7, 0x9A, 0x4B, 0xA5}, {0xF2, 0x1E, 0xD8, 0x38},
			{0x09, 0x3C, 0xD8, 0xF4}, {0x2D, 0xB6, 0xBC, 0x93}, {0x32, 0xAE, 0x12, 0x9E}, {0xAE, 0x79, 0x54, 0xAD},
			{0xB3, 0xEE, 0x0C, 0xFB}, {0xA0, 0xDE, 0x4E, 0xB4}, {0x82, 0xCF, 0x30, 0xFB}, {0x50, 0xC7, 0x79, 0xC0},
			{0x38, 0x94, 0xB2, 0x1C}, {0xB2, 0xA6, 0xD6, 0x73}, {0xA4, 0x4B, 0x34, 0x5C}, {0x4B, 0xC3, 0xCA, 0xE5},
			{0x99, 0x4C, 0xB4, 0x46}, {0x14, 0xB1, 0xC7, 0xB7}, {0x6B, 0x25, 0x0F, 0x4B}, {0x38, 0x13, 0xD4, 0x21},
			{0x5A, 0xCB, 0x8C, 0x1E}, {0x8F, 0x02, 0xB7, 0x21}, {0x7E, 0xF8, 0xC2, 0x48}, {0x5F, 0x30, 0xF6, 0x5B},
			{0x3D, 0x4D, 0x62, 0xFE}, {0x51, 0xF6, 0xD4, 0x14}, {0x2E, 0xBF, 0x0D, 0x87}, {0xFD, 0x06, 0x5A, 0xAA},
			{0xD5, 0x41, 0xF7, 0xB0}, {0x24, 0x8D, 0x19, 0x08}, {0xAF, 0x1A, 0xB6, 0x4A}, {0x0F, 0x70, 0xAC, 0xAF},
			{0x7B, 0x49, 0x47, 0x43}, {0x2D, 0x17, 0xA5, 0x0C}, {0xE4, 0x53, 0x39, 0xD7}, {0xE0, 0xC6, 0x5F, 0xAD},
			{0x78, 0x0B, 0x85, 0xD3}, {0x01, 0x19, 0x48, 0x86}, {0x42, 0xA8, 0xFB, 0xEC}, {0x5B, 0xDB, 0xB2, 0x18},
			{0x68, 0xE4, 0x75, 0xF6}, {0x45, 0x82, 0x5A, 0x23}, {0xF6, 0x9E, 0x7C, 0x94}, {0xF3, 0x08, 0x1C, 0x7F},
			{0xCD, 0x4F, 0x6B, 0xED}, {0x8B, 0x99, 0x92, 0x05}, {0x00, 0x2D, 0x0D, 0xA5}, {0x7E, 0x51, 0x65, 0x6C},
			{0xDD, 0x18, 0xAC, 0xB6}, {0xB5, 0xAC, 0x36, 0x2F}, {0x5D, 0xF5, 0x03, 0xE8}, {0xCE, 0x95, 0x99, 0x7D},
			{0xF6, 0x73, 0xF4, 0x3B}, {0x84, 0x11, 0xED, 0x5D}, {0xAF, 0x4C, 0x18, 0x85}, {0x90, 0x18, 0x3C, 0xFC},
			{0xB9, 0xCF, 0x7D, 0xFF}, {0x1B, 0xC2, 0x65, 0x7A}, {0x13, 0xD8, 0x39, 0x2E}, {0x37, 0x8F, 0xFE, 0xA2},
			{0x48, 0x8A, 0x56, 0xD0}, {0xAC, 0xBA, 0x26, 0x6A}, {0xA6, 0xB7, 0x08, 0x23}, {0x50, 0x94, 0x0A, 0x2D},
			{0xBF, 0x80, 0x08, 0x04}, {0x67, 0x44, 0xEF, 0x60}, {0xF3, 0x40, 0xB5, 0xEE}, {0x5E, 0xAC, 0xBA, 0x82},
			{0xC5, 0x23, 0x3D, 0x4E}, {0xDC, 0x42, 0x90, 0x67}, {0x68, 0xB4, 0x47, 0x69}, {0x9E, 0x8F, 0x8C, 0x2D},
			{0x8B, 0xB6, 0xEA, 0xA0}, {0xFC, 0x2E, 0x68, 0x3F}, {0x66, 0x42, 0x65, 0xC4}, {0x7B, 0xB9, 0xC5, 0xA4},
			{0x8B, 0x8E, 0x00, 0x08}, {0xD7, 0x21, 0x77, 0xC0}, {0x3F, 0xB7, 0x8D, 0xF2}, {0x3E, 0x9E, 0x18, 0x37},
			{0x51, 0x84, 0xA9, 0xEB}, {0x29, 0xFB, 0x54, 0x66}, {0x7A, 0x93, 0xEF, 0xA3}, {0x20, 0xA4, 0x11, 0x44},
			{0xE7, 0x03, 0xB9, 0x9E}, {0x9C, 0xC2, 0x4C, 0xA5}, {0x21, 0x82, 0xBC, 0xBE}, {0x29, 0xF1, 0x3B, 0xCD},
			{0xAA, 0x72, 0x51, 0xB0}, {0xC1, 0xEC, 0xDE, 0x28}, {0xBF, 0x03, 0x9F, 0x17}};
	static uint8_t C[] = {
			0,   1,   2,   3,   4,   5,   6,   7,   8,   9,   10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
			23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,
			46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,
			69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,
			92,  93,  94,  95,  96,  97,  98,  99,  100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114,
			115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137,
			138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159, 160,
			161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175, 176, 177, 178, 179, 180, 181, 182, 183,
			184, 185, 186, 187, 188, 189, 190, 191, 192, 193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 203, 204, 205, 206,
			207, 208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222, 223, 224, 225, 226, 227, 228, 229,
			230, 231, 232, 233, 234, 235, 236, 237, 238, 239, 240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250};
	unsigned mac_len = 4;
	uint8_t adata_mask = 0xE3;
	// Test decrypt for the whole BLE packet len range 1-251
	for (unsigned i = 1; i < 252; i++) {
		P[i - 1] = EncryptedP[i - 1];
		memcpy(&P[i], MAC_VALS[i - 1], mac_len);
		printk("Decrypt packet size %u: ", i);
		build_ccm_test_case_and_run(K, N, P, i + mac_len + 8, i + mac_len, A, sizeof(A), C, i, mac_len, adata_mask, false,
				false, DECRYPT, BLE_MODE, NULL, false);
	}
}

ZTEST(nrf_ccm_tests, test_ccm_02_decr_ble_1)
{
	printk("Test Decrypt 1 (BLE mode) \n");

	uint8_t K[16] = {0xd9, 0x6, 0x2, 0x79, 0x20, 0x98, 0x41, 0x8b, 0x6, 0x3b, 0xb3, 0xb5, 0xe0, 0xa6, 0x47, 0x85};

	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x14, 0xb9, 0xe0, 0xe3, 0xa2, 0x26, 0xe1, 0x65};
	uint8_t A[] = {0xf};
	uint8_t P[] = {0x3b, 0xc5, 0xf3, 0xa7, 0x50};
	uint8_t C[] = {0x6};
	unsigned mac_len = 4;
	uint8_t adata_mask = 0xE3;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, DECRYPT, BLE_MODE, NULL, false);
	//Mismatch, in real HW: CCM EVENTS_ERROR is 1 but expected 0!
}

ZTEST(nrf_ccm_tests, test_ccm_03_decr_out_buf_ovf_2to0_nomac_err) {
	printk("Test for buffer overflow on OUTPTR: Decrypt 2 into 0 byte buffer, expect no MAC error.\n");

	uint8_t K[16] = {0x99, 0xAD, 0x1B, 0x52, 0x26, 0xA3, 0x7E, 0x3E, 0x05, 0x8E, 0x3B, 0x8E, 0x27, 0xC2, 0xC6, 0x66};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
	uint8_t A[] = {0x0E};
	uint8_t P[] = {0x6D, 0x71, 0x65, 0x2C, 0xF0, 0xE7};
	uint8_t C[] = {0x0, 0x1};
	unsigned mac_len = 4;
	uint8_t adata_mask = 0xE3;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, 0, mac_len, adata_mask, false, false,
				DECRYPT, BLE_MODE, NULL, true);
	//Mismatch, in real HW: CCM EVENTS_ERROR is 1 but expected 0!
}

ZTEST(nrf_ccm_tests, test_ccm_04_decr_out_buf_ovf_2to0_mac_err) {
	printk("Test for buffer overflow on OUTPTR: Decrypt 2 into 0 byte buffer, expect MAC error.\n");

	uint8_t K[16] = {0x99, 0xAD, 0x1B, 0x52, 0x26, 0xA3, 0x7E, 0x3E, 0x05, 0x8E, 0x3B, 0x8E, 0x27, 0xC2, 0xC6, 0x66};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
	uint8_t A[] = {0x0E};
	uint8_t P[] = {0x6D, 0x71, 0x65, 0x2C, 0xF0, 0xE8}; // last byte of MAC is wrong!
	uint8_t C[] = {0x0, 0x1};
	unsigned mac_len = 4;
	uint8_t adata_mask = 0xE3;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, 0, mac_len, adata_mask, false,
				EXPECT_MAC_ERROR, DECRYPT, BLE_MODE, NULL, true);
}

ZTEST(nrf_ccm_tests, test_ccm_05_decr_out_buf_ovf_2to1_nomac_err) {
	printk("Test for buffer overflow on OUTPTR: Decrypt 2 into 1 byte buffer, expect no MAC error.\n");

	uint8_t K[16] = {0x99, 0xAD, 0x1B, 0x52, 0x26, 0xA3, 0x7E, 0x3E, 0x05, 0x8E, 0x3B, 0x8E, 0x27, 0xC2, 0xC6, 0x66};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
	uint8_t A[] = {0x0E};
	uint8_t P[] = {0x6D, 0x71, 0x65, 0x2C, 0xF0, 0xE7};
	uint8_t C[] = {0x0, 0x1};
	unsigned mac_len = 4;
	uint8_t adata_mask = 0xE3;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, 1, mac_len, adata_mask, false, false,
			DECRYPT, BLE_MODE, NULL, true);
}

ZTEST(nrf_ccm_tests, test_ccm_06_decr_out_buf_ovf_2to1_mac_err) {
	printk("Test for buffer overflow on OUTPTR: Decrypt 2 into 1 byte buffer, expect MAC error.\n");

	uint8_t K[16] = {0x99, 0xAD, 0x1B, 0x52, 0x26, 0xA3, 0x7E, 0x3E, 0x05, 0x8E, 0x3B, 0x8E, 0x27, 0xC2, 0xC6, 0x66};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
	uint8_t A[] = {0x0E};
	uint8_t P[] = {0x6D, 0x71, 0x65, 0x2C, 0xF0, 0xE8}; // last byte of MAC is wrong!
	uint8_t C[] = {0x0, 0x1};
	unsigned mac_len = 4;
	uint8_t adata_mask = 0xE3;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, 1, mac_len, adata_mask, false,
			EXPECT_MAC_ERROR, DECRYPT, BLE_MODE, NULL, true);
}

ZTEST(nrf_ccm_tests, test_ccm_07_decr_out_buf_1_extra)
{
	printk("Test Decrypt 1 (BLE mode): buffer 1 byte larger than data\n");

	uint8_t K[16] = {0xd9, 0x6, 0x2, 0x79, 0x20, 0x98, 0x41, 0x8b, 0x6, 0x3b, 0xb3, 0xb5, 0xe0, 0xa6, 0x47, 0x85};

	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x14, 0xb9, 0xe0, 0xe3, 0xa2, 0x26, 0xe1, 0x65};
	uint8_t A[] = {0xf};
	uint8_t P[6] = {0x3b, 0xc5, 0xf3, 0xa7, 0x50};
	uint8_t C[] = {0x6};
	unsigned mac_len = 4;
	uint8_t adata_mask = 0xE3;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), 5, A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false, false,
			DECRYPT, BLE_MODE, NULL, false);
	//Mismatch, in real HW: MACSTATUS 0 is wrong, decrypt is 1 expect_macerror is 0!
}

ZTEST(nrf_ccm_tests, test_ccm_08_decr_out_BLE_short_payload) {
	printk("Test Decrypt too short payload in BLE mode\n");

	uint8_t K[16] = {0xd9, 0x6, 0x2, 0x79, 0x20, 0x98, 0x41, 0x8b, 0x6, 0x3b, 0xb3, 0xb5, 0xe0, 0xa6, 0x47, 0x85};

	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x14, 0xb9, 0xe0, 0xe3, 0xa2, 0x26, 0xe1, 0x65};
	uint8_t A[] = {0xf};
	uint8_t P[] = {0x3b, 0xc5};
	uint8_t C[0];
	unsigned mac_len = 4;
	uint8_t adata_mask = 0xE3;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			EXPECT_MAC_ERROR, DECRYPT, BLE_MODE, NULL, false);
}

ZTEST(nrf_ccm_tests, test_ccm_09_decr_out_ieee_short_payload) {
	printk("Test Decrypt too short payload in ieee mode\n");

	uint8_t K[16] = {0xd9, 0x6, 0x2, 0x79, 0x20, 0x98, 0x41, 0x8b, 0x6, 0x3b, 0xb3, 0xb5, 0xe0, 0xa6, 0x47, 0x85};

	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x14, 0xb9, 0xe0, 0xe3, 0xa2, 0x26, 0xe1, 0x65};
	uint8_t A[] = {0xf};
	uint8_t P[] = {0x3b, 0xc5};
	uint8_t C[] = {0x6};
	unsigned mac_len = 4;
	uint8_t adata_mask = 0xE3;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask,
			EXPECT_ERROR, EXPECT_MAC_ERROR, DECRYPT, IEEE_MODE, NULL, false);
}

ZTEST(nrf_ccm_tests, test_ccm_10_decr_out_ble_0_payload) {
	printk("Test Decrypt 0 payload in BLE mode\n");

	uint8_t K[16] = {0x99, 0xAD, 0x1B, 0x52, 0x26, 0xA3, 0x7E, 0x3E, 0x05, 0x8E, 0x3B, 0x8E, 0x27, 0xC2, 0xC6, 0x66};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
	uint8_t A[] = {0x0e};
	uint8_t P[0];
	uint8_t C[0];
	unsigned mac_len = 4;
	uint8_t adata_mask = 0xE3;
	uint16_t mlen_out = 0xffff;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask,
			EXPECT_ERROR, EXPECT_MAC_ERROR, DECRYPT, BLE_MODE, &mlen_out, false);
	zassert_equal(mlen_out, 0, "mlen_out is %u, it should have been 0\n", mlen_out);
}

ZTEST(nrf_ccm_tests, test_ccm_11_decr_out_ieee_0_payload) {
	printk("Test Decrypt 0 payload in ieee mode\n");

	uint8_t K[16] = {0x99, 0xAD, 0x1B, 0x52, 0x26, 0xA3, 0x7E, 0x3E, 0x05, 0x8E, 0x3B, 0x8E, 0x27, 0xC2, 0xC6, 0x66};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
	uint8_t A[] = {0x0e};
	uint8_t P[0];
	uint8_t C[0];
	unsigned mac_len = 4;
	uint8_t adata_mask = 0xE3;
	uint16_t mlen_out = 0xffff;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask,
			EXPECT_ERROR, EXPECT_MAC_ERROR, DECRYPT, IEEE_MODE, &mlen_out, false);
	zassert_equal(mlen_out, 0, "mlen_out is %u, it should have been 0\n", mlen_out);
}

ZTEST(nrf_ccm_tests, test_ccm_12_encr_ble_255b_w_adata)
{
	printk("Encrypt 255 byte payload with adata in BLE mode\n"); // hardware will return success but not encrypt
	// anything and return a 0 len payload

	uint8_t K[16] = {0x99, 0xAD, 0x1B, 0x52, 0x26, 0xA3, 0x7E, 0x3E, 0x05, 0x8E, 0x3B, 0x8E, 0x27, 0xC2, 0xC6, 0x66};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
	uint8_t A[] = {0x0E};
	uint8_t P[] = {0x0,  0x1,  0x2,  0x3,  0x4,  0x5,  0x6,  0x7,  0x8,  0x9,  0xa,  0xb,  0xc,  0xd,  0xe,  0xf,  0x10,
			0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f, 0x20, 0x21,
			0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f, 0x30, 0x31, 0x32,
			0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e, 0x3f, 0x40, 0x41, 0x42, 0x43,
			0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f, 0x50, 0x51, 0x52, 0x53, 0x54,
			0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, 0x5b, 0x5c, 0x5d, 0x5e, 0x5f, 0x60, 0x61, 0x62, 0x63, 0x64, 0x65,
			0x66, 0x67, 0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f, 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76,
			0x77, 0x78, 0x79, 0x7a, 0x7b, 0x7c, 0x7d, 0x7e, 0x7f, 0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87,
			0x88, 0x89, 0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f, 0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98,
			0x99, 0x9a, 0x9b, 0x9c, 0x9d, 0x9e, 0x9f, 0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9,
			0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf, 0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9, 0xba,
			0xbb, 0xbc, 0xbd, 0xbe, 0xbf, 0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xcb,
			0xcc, 0xcd, 0xce, 0xcf, 0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9, 0xda, 0xdb, 0xdc,
			0xdd, 0xde, 0xdf, 0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea, 0xeb, 0xec, 0xed,
			0xee, 0xef, 0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0xfa};
	uint8_t C[] = {
			0x6D, 0x71, 0xB7, 0x03, 0x74, 0x41, 0x0C, 0x9D, 0x0A, 0x74, 0xE2, 0xFD, 0x67, 0x67, 0x64, 0x26, 0x0A, 0xF6, 0x76,
			0x6F, 0x59, 0x4B, 0x7B, 0xBC, 0xC6, 0xDF, 0x18, 0x5B, 0x50, 0x2D, 0x3D, 0x5E, 0xD9, 0xAF, 0xF6, 0xB8, 0xFE, 0x86,
			0xC5, 0x23, 0x3A, 0xA8, 0x35, 0xE4, 0xE6, 0xA0, 0xB6, 0xC9, 0x48, 0xFF, 0xFF, 0xF5, 0xA0, 0xC0, 0x19, 0x32, 0x71,
			0x60, 0x61, 0x0C, 0x1B, 0xE5, 0x43, 0xCE, 0xB5, 0xBC, 0x9E, 0x3A, 0x3D, 0x85, 0xB7, 0xAA, 0x65, 0x64, 0x2E, 0x38,
			0x82, 0x27, 0x3D, 0xA4, 0x13, 0xFE, 0xE5, 0x89, 0x1F, 0x2F, 0x0E, 0x9B, 0x02, 0x0B, 0x10, 0xCD, 0xA8, 0xCD, 0xBA,
			0xA6, 0xD0, 0x40, 0x44, 0xD0, 0x75, 0x37, 0xBB, 0x06, 0x16, 0x54, 0x04, 0x7F, 0x8A, 0x0C, 0x2E, 0x7E, 0xB8, 0x06,
			0x06, 0x63, 0x3F, 0x4A, 0x08, 0xFF, 0x21, 0x28, 0x6C, 0x8F, 0x65, 0xB6, 0x29, 0xE5, 0xFC, 0x58, 0x29, 0x7D, 0x55,
			0xFE, 0xA8, 0x24, 0xDE, 0xE5, 0xB1, 0x8C, 0x9B, 0xAF, 0xD3, 0x78, 0xCE, 0x87, 0x92, 0x07, 0x4F, 0x40, 0x75, 0xE6,
			0xA7, 0x61, 0x09, 0x73, 0xBE, 0x0A, 0x29, 0x05, 0x3F, 0xF7, 0xBD, 0xD8, 0x9C, 0xC7, 0xDA, 0x4B, 0xAC, 0xE9, 0xC2,
			0xE4, 0xD7, 0x4C, 0x2C, 0x26, 0x8C, 0x00, 0xE4, 0xE6, 0xBE, 0x05, 0x0E, 0xC2, 0xDA, 0x1E, 0xA7, 0xAE, 0xD7, 0x35,
			0xA8, 0x26, 0xCD, 0xF8, 0x93, 0x93, 0x26, 0x0C, 0xD5, 0x47, 0x9B, 0x7C, 0x35, 0x62, 0x7D, 0xFB, 0xCF, 0xAD, 0x78,
			0x91, 0xA6, 0x0C, 0xC8, 0x5C, 0xBF, 0x05, 0xBF, 0x7E, 0xE4, 0x0C, 0x4A, 0x9E, 0xC9, 0xD2, 0x9F, 0x75, 0x7A, 0x58,
			0x84, 0x9B, 0xD4, 0x9A, 0x2B, 0x51, 0xB2, 0x4A, 0x3A, 0x45, 0xEB, 0x58, 0x33, 0x86, 0x53, 0xC5, 0xBA, 0x63, 0x73,
			0xB6, 0x89, 0x25, 0xD7, 0xBF, 0x03, 0x9F, 0x17,
	};
	unsigned mac_len = 4;
	uint8_t adata_mask = 0xE3;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);
}

ZTEST(nrf_ccm_tests, test_ccm_12b_encr_ble_255b_w_adata_nomac)
{
	printk("Encrypt 255 byte payload with adata and no MAC in BLE mode\n");

	uint8_t K[16] = {0x99, 0xAD, 0x1B, 0x52, 0x26, 0xA3, 0x7E, 0x3E, 0x05, 0x8E, 0x3B, 0x8E, 0x27, 0xC2, 0xC6, 0x66};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
	uint8_t A[] = {0x0E};
	uint8_t P[] = {0x0,  0x1,  0x2,  0x3,  0x4,  0x5,  0x6,  0x7,  0x8,  0x9,  0xa,  0xb,  0xc,  0xd,  0xe,  0xf,  0x10,
			0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f, 0x20, 0x21,
			0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f, 0x30, 0x31, 0x32,
			0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e, 0x3f, 0x40, 0x41, 0x42, 0x43,
			0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f, 0x50, 0x51, 0x52, 0x53, 0x54,
			0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, 0x5b, 0x5c, 0x5d, 0x5e, 0x5f, 0x60, 0x61, 0x62, 0x63, 0x64, 0x65,
			0x66, 0x67, 0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f, 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76,
			0x77, 0x78, 0x79, 0x7a, 0x7b, 0x7c, 0x7d, 0x7e, 0x7f, 0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87,
			0x88, 0x89, 0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f, 0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98,
			0x99, 0x9a, 0x9b, 0x9c, 0x9d, 0x9e, 0x9f, 0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9,
			0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf, 0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9, 0xba,
			0xbb, 0xbc, 0xbd, 0xbe, 0xbf, 0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xcb,
			0xcc, 0xcd, 0xce, 0xcf, 0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9, 0xda, 0xdb, 0xdc,
			0xdd, 0xde, 0xdf, 0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea, 0xeb, 0xec, 0xed,
			0xee, 0xef, 0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0xfa};
	uint8_t C[] = {
			0x6D, 0x71, 0xB7, 0x03, 0x74, 0x41, 0x0C, 0x9D, 0x0A, 0x74, 0xE2, 0xFD, 0x67, 0x67, 0x64, 0x26, 0x0A, 0xF6, 0x76,
			0x6F, 0x59, 0x4B, 0x7B, 0xBC, 0xC6, 0xDF, 0x18, 0x5B, 0x50, 0x2D, 0x3D, 0x5E, 0xD9, 0xAF, 0xF6, 0xB8, 0xFE, 0x86,
			0xC5, 0x23, 0x3A, 0xA8, 0x35, 0xE4, 0xE6, 0xA0, 0xB6, 0xC9, 0x48, 0xFF, 0xFF, 0xF5, 0xA0, 0xC0, 0x19, 0x32, 0x71,
			0x60, 0x61, 0x0C, 0x1B, 0xE5, 0x43, 0xCE, 0xB5, 0xBC, 0x9E, 0x3A, 0x3D, 0x85, 0xB7, 0xAA, 0x65, 0x64, 0x2E, 0x38,
			0x82, 0x27, 0x3D, 0xA4, 0x13, 0xFE, 0xE5, 0x89, 0x1F, 0x2F, 0x0E, 0x9B, 0x02, 0x0B, 0x10, 0xCD, 0xA8, 0xCD, 0xBA,
			0xA6, 0xD0, 0x40, 0x44, 0xD0, 0x75, 0x37, 0xBB, 0x06, 0x16, 0x54, 0x04, 0x7F, 0x8A, 0x0C, 0x2E, 0x7E, 0xB8, 0x06,
			0x06, 0x63, 0x3F, 0x4A, 0x08, 0xFF, 0x21, 0x28, 0x6C, 0x8F, 0x65, 0xB6, 0x29, 0xE5, 0xFC, 0x58, 0x29, 0x7D, 0x55,
			0xFE, 0xA8, 0x24, 0xDE, 0xE5, 0xB1, 0x8C, 0x9B, 0xAF, 0xD3, 0x78, 0xCE, 0x87, 0x92, 0x07, 0x4F, 0x40, 0x75, 0xE6,
			0xA7, 0x61, 0x09, 0x73, 0xBE, 0x0A, 0x29, 0x05, 0x3F, 0xF7, 0xBD, 0xD8, 0x9C, 0xC7, 0xDA, 0x4B, 0xAC, 0xE9, 0xC2,
			0xE4, 0xD7, 0x4C, 0x2C, 0x26, 0x8C, 0x00, 0xE4, 0xE6, 0xBE, 0x05, 0x0E, 0xC2, 0xDA, 0x1E, 0xA7, 0xAE, 0xD7, 0x35,
			0xA8, 0x26, 0xCD, 0xF8, 0x93, 0x93, 0x26, 0x0C, 0xD5, 0x47, 0x9B, 0x7C, 0x35, 0x62, 0x7D, 0xFB, 0xCF, 0xAD, 0x78,
			0x91, 0xA6, 0x0C, 0xC8, 0x5C, 0xBF, 0x05, 0xBF, 0x7E, 0xE4, 0x0C, 0x4A, 0x9E, 0xC9, 0xD2, 0x9F, 0x75, 0x7A, 0x58,
			0x84, 0x9B, 0xD4, 0x9A, 0x2B, 0x51, 0xB2, 0x4A, 0x3A, 0x45, 0xEB, 0x58, 0x33, 0x86, 0x53, 0xC5, 0xBA, 0x63, 0x73,
			0xB6, 0x89, 0x25, 0xD7,
	};
	unsigned mac_len = 0;
	uint8_t adata_mask = 0xE3;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);
}

ZTEST(nrf_ccm_tests, test_ccm_12c_encr_ble_0_255b_w_adata_nomac)
{
	printk("Encrypt 0-255 byte payload with adata and no MAC in BLE mode\n");

	uint8_t K[16] = {0x99, 0xAD, 0x1B, 0x52, 0x26, 0xA3, 0x7E, 0x3E, 0x05, 0x8E, 0x3B, 0x8E, 0x27, 0xC2, 0xC6, 0x66};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
	uint8_t A[] = {0x0E};
	uint8_t P[] = {0x0,  0x1,  0x2,  0x3,  0x4,  0x5,  0x6,  0x7,  0x8,  0x9, 0xa, 0xb,  0xc,  0xd,  0xe,  0xf,  0x10,
			0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f, 0x20, 0x21,
			0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f, 0x30, 0x31, 0x32,
			0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e, 0x3f, 0x40, 0x41, 0x42, 0x43,
			0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f, 0x50, 0x51, 0x52, 0x53, 0x54,
			0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, 0x5b, 0x5c, 0x5d, 0x5e, 0x5f, 0x60, 0x61, 0x62, 0x63, 0x64, 0x65,
			0x66, 0x67, 0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f, 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76,
			0x77, 0x78, 0x79, 0x7a, 0x7b, 0x7c, 0x7d, 0x7e, 0x7f, 0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87,
			0x88, 0x89, 0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f, 0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98,
			0x99, 0x9a, 0x9b, 0x9c, 0x9d, 0x9e, 0x9f, 0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9,
			0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf, 0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9, 0xba,
			0xbb, 0xbc, 0xbd, 0xbe, 0xbf, 0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xcb,
			0xcc, 0xcd, 0xce, 0xcf, 0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9, 0xda, 0xdb, 0xdc,
			0xdd, 0xde, 0xdf, 0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea, 0xeb, 0xec, 0xed,
			0xee, 0xef, 0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0xfa};
	uint8_t C[] = {
			0x6D, 0x71, 0xB7, 0x03, 0x74, 0x41, 0x0C, 0x9D, 0x0A, 0x74, 0xE2, 0xFD, 0x67, 0x67, 0x64, 0x26, 0x0A, 0xF6, 0x76,
			0x6F, 0x59, 0x4B, 0x7B, 0xBC, 0xC6, 0xDF, 0x18, 0x5B, 0x50, 0x2D, 0x3D, 0x5E, 0xD9, 0xAF, 0xF6, 0xB8, 0xFE, 0x86,
			0xC5, 0x23, 0x3A, 0xA8, 0x35, 0xE4, 0xE6, 0xA0, 0xB6, 0xC9, 0x48, 0xFF, 0xFF, 0xF5, 0xA0, 0xC0, 0x19, 0x32, 0x71,
			0x60, 0x61, 0x0C, 0x1B, 0xE5, 0x43, 0xCE, 0xB5, 0xBC, 0x9E, 0x3A, 0x3D, 0x85, 0xB7, 0xAA, 0x65, 0x64, 0x2E, 0x38,
			0x82, 0x27, 0x3D, 0xA4, 0x13, 0xFE, 0xE5, 0x89, 0x1F, 0x2F, 0x0E, 0x9B, 0x02, 0x0B, 0x10, 0xCD, 0xA8, 0xCD, 0xBA,
			0xA6, 0xD0, 0x40, 0x44, 0xD0, 0x75, 0x37, 0xBB, 0x06, 0x16, 0x54, 0x04, 0x7F, 0x8A, 0x0C, 0x2E, 0x7E, 0xB8, 0x06,
			0x06, 0x63, 0x3F, 0x4A, 0x08, 0xFF, 0x21, 0x28, 0x6C, 0x8F, 0x65, 0xB6, 0x29, 0xE5, 0xFC, 0x58, 0x29, 0x7D, 0x55,
			0xFE, 0xA8, 0x24, 0xDE, 0xE5, 0xB1, 0x8C, 0x9B, 0xAF, 0xD3, 0x78, 0xCE, 0x87, 0x92, 0x07, 0x4F, 0x40, 0x75, 0xE6,
			0xA7, 0x61, 0x09, 0x73, 0xBE, 0x0A, 0x29, 0x05, 0x3F, 0xF7, 0xBD, 0xD8, 0x9C, 0xC7, 0xDA, 0x4B, 0xAC, 0xE9, 0xC2,
			0xE4, 0xD7, 0x4C, 0x2C, 0x26, 0x8C, 0x00, 0xE4, 0xE6, 0xBE, 0x05, 0x0E, 0xC2, 0xDA, 0x1E, 0xA7, 0xAE, 0xD7, 0x35,
			0xA8, 0x26, 0xCD, 0xF8, 0x93, 0x93, 0x26, 0x0C, 0xD5, 0x47, 0x9B, 0x7C, 0x35, 0x62, 0x7D, 0xFB, 0xCF, 0xAD, 0x78,
			0x91, 0xA6, 0x0C, 0xC8, 0x5C, 0xBF, 0x05, 0xBF, 0x7E, 0xE4, 0x0C, 0x4A, 0x9E, 0xC9, 0xD2, 0x9F, 0x75, 0x7A, 0x58,
			0x84, 0x9B, 0xD4, 0x9A, 0x2B, 0x51, 0xB2, 0x4A, 0x3A, 0x45, 0xEB, 0x58, 0x33, 0x86, 0x53, 0xC5, 0xBA, 0x63, 0x73,
			0xB6, 0x89, 0x25, 0xD7,
	};
	unsigned mac_len = 0;
	uint8_t adata_mask = 0xE3;

	for (uint8_t len = 0; len <=250; len++)
	{
		printk("Encrypt %d bytes payload\n", len);
		build_ccm_test_case_and_run(K, N, P, len, len, A, sizeof(A), C, len, mac_len, adata_mask, false,
				false, ENCRYPT, BLE_MODE, NULL, false);
	}
}

ZTEST(nrf_ccm_tests, test_ccm_13_decr_ble_255b_w_adata_wmac)
{
	printk("Test Decrypt 255 payload with MAC in BLE mode\n");

	uint8_t K[16] = {0x99, 0xAD, 0x1B, 0x52, 0x26, 0xA3, 0x7E, 0x3E, 0x05, 0x8E, 0x3B, 0x8E, 0x27, 0xC2, 0xC6, 0x66};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
	uint8_t A[] = {0x0e};
	uint8_t P[] = {
			0x6D, 0x71, 0xB7, 0x03, 0x74, 0x41, 0x0C, 0x9D, 0x0A, 0x74, 0xE2, 0xFD, 0x67, 0x67, 0x64, 0x26, 0x0A, 0xF6, 0x76,
			0x6F, 0x59, 0x4B, 0x7B, 0xBC, 0xC6, 0xDF, 0x18, 0x5B, 0x50, 0x2D, 0x3D, 0x5E, 0xD9, 0xAF, 0xF6, 0xB8, 0xFE, 0x86,
			0xC5, 0x23, 0x3A, 0xA8, 0x35, 0xE4, 0xE6, 0xA0, 0xB6, 0xC9, 0x48, 0xFF, 0xFF, 0xF5, 0xA0, 0xC0, 0x19, 0x32, 0x71,
			0x60, 0x61, 0x0C, 0x1B, 0xE5, 0x43, 0xCE, 0xB5, 0xBC, 0x9E, 0x3A, 0x3D, 0x85, 0xB7, 0xAA, 0x65, 0x64, 0x2E, 0x38,
			0x82, 0x27, 0x3D, 0xA4, 0x13, 0xFE, 0xE5, 0x89, 0x1F, 0x2F, 0x0E, 0x9B, 0x02, 0x0B, 0x10, 0xCD, 0xA8, 0xCD, 0xBA,
			0xA6, 0xD0, 0x40, 0x44, 0xD0, 0x75, 0x37, 0xBB, 0x06, 0x16, 0x54, 0x04, 0x7F, 0x8A, 0x0C, 0x2E, 0x7E, 0xB8, 0x06,
			0x06, 0x63, 0x3F, 0x4A, 0x08, 0xFF, 0x21, 0x28, 0x6C, 0x8F, 0x65, 0xB6, 0x29, 0xE5, 0xFC, 0x58, 0x29, 0x7D, 0x55,
			0xFE, 0xA8, 0x24, 0xDE, 0xE5, 0xB1, 0x8C, 0x9B, 0xAF, 0xD3, 0x78, 0xCE, 0x87, 0x92, 0x07, 0x4F, 0x40, 0x75, 0xE6,
			0xA7, 0x61, 0x09, 0x73, 0xBE, 0x0A, 0x29, 0x05, 0x3F, 0xF7, 0xBD, 0xD8, 0x9C, 0xC7, 0xDA, 0x4B, 0xAC, 0xE9, 0xC2,
			0xE4, 0xD7, 0x4C, 0x2C, 0x26, 0x8C, 0x00, 0xE4, 0xE6, 0xBE, 0x05, 0x0E, 0xC2, 0xDA, 0x1E, 0xA7, 0xAE, 0xD7, 0x35,
			0xA8, 0x26, 0xCD, 0xF8, 0x93, 0x93, 0x26, 0x0C, 0xD5, 0x47, 0x9B, 0x7C, 0x35, 0x62, 0x7D, 0xFB, 0xCF, 0xAD, 0x78,
			0x91, 0xA6, 0x0C, 0xC8, 0x5C, 0xBF, 0x05, 0xBF, 0x7E, 0xE4, 0x0C, 0x4A, 0x9E, 0xC9, 0xD2, 0x9F, 0x75, 0x7A, 0x58,
			0x84, 0x9B, 0xD4, 0x9A, 0x2B, 0x51, 0xB2, 0x4A, 0x3A, 0x45, 0xEB, 0x58, 0x33, 0x86, 0x53, 0xC5, 0xBA, 0x63, 0x73,
			0xB6, 0x89, 0x25, 0xD7, 0xBF, 0x03, 0x9F, 0x17,
	};
	uint8_t C[] = {0x0,  0x1,  0x2,  0x3,  0x4,  0x5,  0x6,  0x7,  0x8,  0x9,  0xa,  0xb,  0xc,  0xd,  0xe,  0xf,  0x10,
			0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f, 0x20, 0x21,
			0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f, 0x30, 0x31, 0x32,
			0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e, 0x3f, 0x40, 0x41, 0x42, 0x43,
			0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f, 0x50, 0x51, 0x52, 0x53, 0x54,
			0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, 0x5b, 0x5c, 0x5d, 0x5e, 0x5f, 0x60, 0x61, 0x62, 0x63, 0x64, 0x65,
			0x66, 0x67, 0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f, 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76,
			0x77, 0x78, 0x79, 0x7a, 0x7b, 0x7c, 0x7d, 0x7e, 0x7f, 0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87,
			0x88, 0x89, 0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f, 0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98,
			0x99, 0x9a, 0x9b, 0x9c, 0x9d, 0x9e, 0x9f, 0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9,
			0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf, 0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9, 0xba,
			0xbb, 0xbc, 0xbd, 0xbe, 0xbf, 0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xcb,
			0xcc, 0xcd, 0xce, 0xcf, 0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9, 0xda, 0xdb, 0xdc,
			0xdd, 0xde, 0xdf, 0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea, 0xeb, 0xec, 0xed,
			0xee, 0xef, 0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0xfa};
	unsigned mac_len = 4;
	uint8_t adata_mask = 0xE3;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, DECRYPT, BLE_MODE, NULL, false);
}

ZTEST(nrf_ccm_tests, test_ccm_13b_decr_ble_255b_w_adata_nomac)
{
	printk("Test Decrypt 255 payload without MAC in BLE mode\n");

	uint8_t K[16] = {0x99, 0xAD, 0x1B, 0x52, 0x26, 0xA3, 0x7E, 0x3E, 0x05, 0x8E, 0x3B, 0x8E, 0x27, 0xC2, 0xC6, 0x66};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
	uint8_t A[] = {0x0e};
	uint8_t P[] = {
			0x6D, 0x71, 0xB7, 0x03, 0x74, 0x41, 0x0C, 0x9D, 0x0A, 0x74, 0xE2, 0xFD, 0x67, 0x67, 0x64, 0x26, 0x0A, 0xF6, 0x76,
			0x6F, 0x59, 0x4B, 0x7B, 0xBC, 0xC6, 0xDF, 0x18, 0x5B, 0x50, 0x2D, 0x3D, 0x5E, 0xD9, 0xAF, 0xF6, 0xB8, 0xFE, 0x86,
			0xC5, 0x23, 0x3A, 0xA8, 0x35, 0xE4, 0xE6, 0xA0, 0xB6, 0xC9, 0x48, 0xFF, 0xFF, 0xF5, 0xA0, 0xC0, 0x19, 0x32, 0x71,
			0x60, 0x61, 0x0C, 0x1B, 0xE5, 0x43, 0xCE, 0xB5, 0xBC, 0x9E, 0x3A, 0x3D, 0x85, 0xB7, 0xAA, 0x65, 0x64, 0x2E, 0x38,
			0x82, 0x27, 0x3D, 0xA4, 0x13, 0xFE, 0xE5, 0x89, 0x1F, 0x2F, 0x0E, 0x9B, 0x02, 0x0B, 0x10, 0xCD, 0xA8, 0xCD, 0xBA,
			0xA6, 0xD0, 0x40, 0x44, 0xD0, 0x75, 0x37, 0xBB, 0x06, 0x16, 0x54, 0x04, 0x7F, 0x8A, 0x0C, 0x2E, 0x7E, 0xB8, 0x06,
			0x06, 0x63, 0x3F, 0x4A, 0x08, 0xFF, 0x21, 0x28, 0x6C, 0x8F, 0x65, 0xB6, 0x29, 0xE5, 0xFC, 0x58, 0x29, 0x7D, 0x55,
			0xFE, 0xA8, 0x24, 0xDE, 0xE5, 0xB1, 0x8C, 0x9B, 0xAF, 0xD3, 0x78, 0xCE, 0x87, 0x92, 0x07, 0x4F, 0x40, 0x75, 0xE6,
			0xA7, 0x61, 0x09, 0x73, 0xBE, 0x0A, 0x29, 0x05, 0x3F, 0xF7, 0xBD, 0xD8, 0x9C, 0xC7, 0xDA, 0x4B, 0xAC, 0xE9, 0xC2,
			0xE4, 0xD7, 0x4C, 0x2C, 0x26, 0x8C, 0x00, 0xE4, 0xE6, 0xBE, 0x05, 0x0E, 0xC2, 0xDA, 0x1E, 0xA7, 0xAE, 0xD7, 0x35,
			0xA8, 0x26, 0xCD, 0xF8, 0x93, 0x93, 0x26, 0x0C, 0xD5, 0x47, 0x9B, 0x7C, 0x35, 0x62, 0x7D, 0xFB, 0xCF, 0xAD, 0x78,
			0x91, 0xA6, 0x0C, 0xC8, 0x5C, 0xBF, 0x05, 0xBF, 0x7E, 0xE4, 0x0C, 0x4A, 0x9E, 0xC9, 0xD2, 0x9F, 0x75, 0x7A, 0x58,
			0x84, 0x9B, 0xD4, 0x9A, 0x2B, 0x51, 0xB2, 0x4A, 0x3A, 0x45, 0xEB, 0x58, 0x33, 0x86, 0x53, 0xC5, 0xBA, 0x63, 0x73,
			0xB6, 0x89, 0x25, 0xD7,
	};
	uint8_t C[] = {0x0,  0x1,  0x2,  0x3,  0x4,  0x5,  0x6,  0x7,  0x8,  0x9,  0xa,  0xb,  0xc,  0xd,  0xe,  0xf,  0x10,
			0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f, 0x20, 0x21,
			0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f, 0x30, 0x31, 0x32,
			0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e, 0x3f, 0x40, 0x41, 0x42, 0x43,
			0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f, 0x50, 0x51, 0x52, 0x53, 0x54,
			0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, 0x5b, 0x5c, 0x5d, 0x5e, 0x5f, 0x60, 0x61, 0x62, 0x63, 0x64, 0x65,
			0x66, 0x67, 0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f, 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76,
			0x77, 0x78, 0x79, 0x7a, 0x7b, 0x7c, 0x7d, 0x7e, 0x7f, 0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87,
			0x88, 0x89, 0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f, 0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98,
			0x99, 0x9a, 0x9b, 0x9c, 0x9d, 0x9e, 0x9f, 0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9,
			0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf, 0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9, 0xba,
			0xbb, 0xbc, 0xbd, 0xbe, 0xbf, 0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xcb,
			0xcc, 0xcd, 0xce, 0xcf, 0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9, 0xda, 0xdb, 0xdc,
			0xdd, 0xde, 0xdf, 0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea, 0xeb, 0xec, 0xed,
			0xee, 0xef, 0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0xfa};
	unsigned mac_len = 0;
	uint8_t adata_mask = 0xE3;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, DECRYPT, BLE_MODE, NULL, false);
}


ZTEST(nrf_ccm_tests, test_ccm_14_decr_ble_0b_w_adata_wmac) {
	printk("Test Decrypt 0 payload with MAC in BLE mode\n");

	uint8_t K[16] = {0x99, 0xAD, 0x1B, 0x52, 0x26, 0xA3, 0x7E, 0x3E, 0x05, 0x8E, 0x3B, 0x8E, 0x27, 0xC2, 0xC6, 0x66};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
	uint8_t A[] = {0x0e};
	uint8_t P[] = {0x1C, 0x32, 0x94, 0x1E};
	uint8_t C[0];
	unsigned mac_len = 4;
	uint8_t adata_mask = 0xE3;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			EXPECT_MAC_ERROR, DECRYPT, BLE_MODE, NULL, false);
}

ZTEST(nrf_ccm_tests, test_ccm_15_decr_ieee_0b_w_adata_wmac) {
	printk("Test Decrypt 0 payload with MAC in ieee mode\n");

	uint8_t K[16] = {0x99, 0xAD, 0x1B, 0x52, 0x26, 0xA3, 0x7E, 0x3E, 0x05, 0x8E, 0x3B, 0x8E, 0x27, 0xC2, 0xC6, 0x66};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
	uint8_t A[] = {0x0e};
	uint8_t P[] = {0x1C, 0x32, 0x94, 0x1E};
	uint8_t C[0];
	unsigned mac_len = 4;
	uint8_t adata_mask = 0xE3;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, DECRYPT, IEEE_MODE, NULL, false);
}

ZTEST(nrf_ccm_tests, test_ccm_16_encr_ble_0b_w_adata3)
{
	printk("Encrypt 0 byte payload with adata in BLE mode\n"); // hardware will return success but not encrypt
	// anything and return a 0 len payload

	uint8_t K[16] = {0x99, 0xAD, 0x1B, 0x52, 0x26, 0xA3, 0x7E, 0x3E, 0x05, 0x8E, 0x3B, 0x8E, 0x27, 0xC2, 0xC6, 0x66};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
	uint8_t A[] = {0x0E};
	uint8_t P[0];
	uint8_t C[0]; // = {0x1C, 0x32, 0x94, 0x1E};
	unsigned mac_len = 4;
	uint8_t adata_mask = 0xE3;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);
}

ZTEST(nrf_ccm_tests, test_ccm_17_encr_HFV)
{
	printk("Testing HFV test vector\n");

	uint8_t K[16] = {0x99, 0xAD, 0x1B, 0x52, 0x26, 0xA3, 0x7E, 0x3E, 0x05, 0x8E, 0x3B, 0x8E, 0x27, 0xC2, 0xC6, 0x66};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x24, 0xAB, 0xDC, 0xBA, 0xBE, 0xBA, 0xAF, 0xDE};
	uint8_t A[] = {0x0E};
	uint8_t P[] = {0x17, 0x00, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e,
			0x6f, 0x70, 0x71, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x30};
	uint8_t C[] = {0x7A, 0x70, 0xD6, 0x64, 0x15, 0x22, 0x6D, 0xF2, 0x6B, 0x17, 0x83, 0x9A, 0x06, 0x04, 0x05, 0x59,
			0x6B, 0xD6, 0x56, 0x4F, 0x79, 0x6B, 0x5B, 0x9C, 0xE6, 0xFF, 0x32, 0xF7, 0x5A, 0x6D, 0x33};
	unsigned mac_len = 4;
	uint8_t adata_mask = 0xE3;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);
}

ZTEST(nrf_ccm_tests, test_ccm_18_encrypt_ble_8mac) {
	// Test vectors from https://www.ietf.org/rfc/rfc3610.txt; generated by parse_rfc3610.txt.py
	printk("Testing Packet Vector #1 in BLE mode\n");

	uint8_t K[16] = {0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x02, 0x01, 0x00, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5};
	uint8_t A[] = {0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7};
	uint8_t P[] = {0x8,  0x9,  0xa,  0xb,  0xc,  0xd,  0xe,  0xf,  0x10, 0x11, 0x12, 0x13,
			0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e};
	uint8_t C[] = {0x58, 0x8c, 0x97, 0x9a, 0x61, 0xc6, 0x63, 0xd2, 0xf0, 0x66, 0xd0, 0xc2, 0xc0, 0xf9, 0x89, 0x80,
			0x6d, 0x5f, 0x6b, 0x61, 0xda, 0xc3, 0x84, 0x17, 0xe8, 0xd1, 0x2c, 0xfd, 0xf9, 0x26, 0xe0};
	unsigned mac_len = 8;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_19_encrypt_ieee_8mac) {
	printk("Testing Packet Vector #1 in ieee mode\n");

	uint8_t K[16] = {0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x02, 0x01, 0x00, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5};
	uint8_t A[] = {0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7};
	uint8_t P[] = {0x8,  0x9,  0xa,  0xb,  0xc,  0xd,  0xe,  0xf,  0x10, 0x11, 0x12, 0x13,
			0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e};
	uint8_t C[] = {0x58, 0x8c, 0x97, 0x9a, 0x61, 0xc6, 0x63, 0xd2, 0xf0, 0x66, 0xd0, 0xc2, 0xc0, 0xf9, 0x89, 0x80,
			0x6d, 0x5f, 0x6b, 0x61, 0xda, 0xc3, 0x84, 0x17, 0xe8, 0xd1, 0x2c, 0xfd, 0xf9, 0x26, 0xe0};
	unsigned mac_len = 8;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, IEEE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_20_encrypt_ble_8mac_2) {
	printk("Testing Packet Vector #2\n");

	uint8_t K[16] = {0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x03, 0x02, 0x01, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5};
	uint8_t A[] = {0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7};
	uint8_t P[] = {0x8,  0x9,  0xa,  0xb,  0xc,  0xd,  0xe,  0xf,  0x10, 0x11, 0x12, 0x13,
			0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f};
	uint8_t C[] = {0x72, 0xc9, 0x1a, 0x36, 0xe1, 0x35, 0xf8, 0xcf, 0x29, 0x1c, 0xa8, 0x94, 0x8,  0x5c, 0x87, 0xe3,
			0xcc, 0x15, 0xc4, 0x39, 0xc9, 0xe4, 0x3a, 0x3b, 0xa0, 0x91, 0xd5, 0x6e, 0x10, 0x40, 0x9,  0x16};
	unsigned mac_len = 8;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_21_encrypt_ble_8mac_3) {
	printk("Testing Packet Vector #3\n");

	uint8_t K[16] = {0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x05, 0x04, 0x03, 0x02, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5};
	uint8_t A[] = {0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7};
	uint8_t P[] = {0x8,  0x9,  0xa,  0xb,  0xc,  0xd,  0xe,  0xf,  0x10, 0x11, 0x12, 0x13, 0x14,
			0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f, 0x20};
	uint8_t C[] = {0x51, 0xb1, 0xe5, 0xf4, 0x4a, 0x19, 0x7d, 0x1d, 0xa4, 0x6b, 0xf,  0x8e, 0x2d, 0x28, 0x2a, 0xe8, 0x71,
			0xe8, 0x38, 0xbb, 0x64, 0xda, 0x85, 0x96, 0x57, 0x4a, 0xda, 0xa7, 0x6f, 0xbd, 0x9f, 0xb0, 0xc5};
	unsigned mac_len = 8;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_22_encrypt_ble_8mac_4) {
	printk("Testing Packet Vector #4\n");

	uint8_t K[16] = {0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x05, 0x04, 0x03, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5};
	uint8_t A[] = {0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa, 0xb};
	uint8_t P[] = {0xc,  0xd,  0xe,  0xf,  0x10, 0x11, 0x12, 0x13, 0x14, 0x15,
			0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e};
	uint8_t C[] = {0xa2, 0x8c, 0x68, 0x65, 0x93, 0x9a, 0x9a, 0x79, 0xfa, 0xaa, 0x5c, 0x4c, 0x2a, 0x9d,
			0x4a, 0x91, 0xcd, 0xac, 0x8c, 0x96, 0xc8, 0x61, 0xb9, 0xc9, 0xe6, 0x1e, 0xf1};
	unsigned mac_len = 8;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_23_encrypt_ble_8mac_5) {
	printk("Testing Packet Vector #5\n");

	uint8_t K[16] = {0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x07, 0x06, 0x05, 0x04, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5};
	uint8_t A[] = {0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa, 0xb};
	uint8_t P[] = {0xc,  0xd,  0xe,  0xf,  0x10, 0x11, 0x12, 0x13, 0x14, 0x15,
			0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f};
	uint8_t C[] = {0xdc, 0xf1, 0xfb, 0x7b, 0x5d, 0x9e, 0x23, 0xfb, 0x9d, 0x4e, 0x13, 0x12, 0x53, 0x65,
			0x8a, 0xd8, 0x6e, 0xbd, 0xca, 0x3e, 0x51, 0xe8, 0x3f, 0x7,  0x7d, 0x9c, 0x2d, 0x93};
	unsigned mac_len = 8;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_24_encrypt_ble_8mac_6) {
	printk("Testing Packet Vector #6\n");

	uint8_t K[16] = {0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x07, 0x06, 0x05, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5};
	uint8_t A[] = {0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa, 0xb};
	uint8_t P[] = {0xc,  0xd,  0xe,  0xf,  0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16,
			0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f, 0x20};
	uint8_t C[] = {0x6f, 0xc1, 0xb0, 0x11, 0xf0, 0x6,  0x56, 0x8b, 0x51, 0x71, 0xa4, 0x2d, 0x95, 0x3d, 0x46,
			0x9b, 0x25, 0x70, 0xa4, 0xbd, 0x87, 0x40, 0x5a, 0x4,  0x43, 0xac, 0x91, 0xcb, 0x94};
	unsigned mac_len = 8;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_25_encrypt_ble_10mac_7) {
	printk("Testing Packet Vector #7\n");

	uint8_t K[16] = {0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x09, 0x08, 0x07, 0x06, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5};
	uint8_t A[] = {0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7};
	uint8_t P[] = {0x8,  0x9,  0xa,  0xb,  0xc,  0xd,  0xe,  0xf,  0x10, 0x11, 0x12, 0x13,
			0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e};
	uint8_t C[] = {0x1,  0x35, 0xd1, 0xb2, 0xc9, 0x5f, 0x41, 0xd5, 0xd1, 0xd4, 0xfe, 0xc1, 0x85, 0xd1, 0x66, 0xb8, 0x9,
			0x4e, 0x99, 0x9d, 0xfe, 0xd9, 0x6c, 0x4,  0x8c, 0x56, 0x60, 0x2c, 0x97, 0xac, 0xbb, 0x74, 0x90};
	unsigned mac_len = 10;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_26_encrypt_ble_10mac_8) {
	printk("Testing Packet Vector #8\n");

	uint8_t K[16] = {0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0A, 0x09, 0x08, 0x07, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5};
	uint8_t A[] = {0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7};
	uint8_t P[] = {0x8,  0x9,  0xa,  0xb,  0xc,  0xd,  0xe,  0xf,  0x10, 0x11, 0x12, 0x13,
			0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f};
	uint8_t C[] = {0x7b, 0x75, 0x39, 0x9a, 0xc0, 0x83, 0x1d, 0xd2, 0xf0, 0xbb, 0xd7, 0x58,
			0x79, 0xa2, 0xfd, 0x8f, 0x6c, 0xae, 0x6b, 0x6c, 0xd9, 0xb7, 0xdb, 0x24,
			0xc1, 0x7b, 0x44, 0x33, 0xf4, 0x34, 0x96, 0x3f, 0x34, 0xb4};
	unsigned mac_len = 10;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_27_encrypt_ble_10mac_9) {
	printk("Testing Packet Vector #9\n");

	uint8_t K[16] = {0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0B, 0x0A, 0x09, 0x08, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5};
	uint8_t A[] = {0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7};
	uint8_t P[] = {0x8,  0x9,  0xa,  0xb,  0xc,  0xd,  0xe,  0xf,  0x10, 0x11, 0x12, 0x13, 0x14,
			0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f, 0x20};
	uint8_t C[] = {0x82, 0x53, 0x1a, 0x60, 0xcc, 0x24, 0x94, 0x5a, 0x4b, 0x82, 0x79, 0x18,
			0x1a, 0xb5, 0xc8, 0x4d, 0xf2, 0x1c, 0xe7, 0xf9, 0xb7, 0x3f, 0x42, 0xe1,
			0x97, 0xea, 0x9c, 0x7,  0xe5, 0x6b, 0x5e, 0xb1, 0x7e, 0x5f, 0x4e};
	unsigned mac_len = 10;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_28_encrypt_ble_10mac_10) {
	printk("Testing Packet Vector #10\n");

	uint8_t K[16] = {0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0C, 0x0B, 0x0A, 0x09, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5};
	uint8_t A[] = {0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa, 0xb};
	uint8_t P[] = {0xc,  0xd,  0xe,  0xf,  0x10, 0x11, 0x12, 0x13, 0x14, 0x15,
			0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e};
	uint8_t C[] = {0x7,  0x34, 0x25, 0x94, 0x15, 0x77, 0x85, 0x15, 0x2b, 0x7,  0x40, 0x98, 0x33, 0xa, 0xbb,
			0x14, 0x1b, 0x94, 0x7b, 0x56, 0x6a, 0xa9, 0x40, 0x6b, 0x4d, 0x99, 0x99, 0x88, 0xdd};
	unsigned mac_len = 10;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_29_encrypt_ble_10mac_11) {
	printk("Testing Packet Vector #11\n");

	uint8_t K[16] = {0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0D, 0x0C, 0x0B, 0x0A, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5};
	uint8_t A[] = {0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa, 0xb};
	uint8_t P[] = {0xc,  0xd,  0xe,  0xf,  0x10, 0x11, 0x12, 0x13, 0x14, 0x15,
			0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f};
	uint8_t C[] = {0x67, 0x6b, 0xb2, 0x3,  0x80, 0xb0, 0xe3, 0x1,  0xe8, 0xab, 0x79, 0x59, 0xa,  0x39, 0x6d,
			0xa7, 0x8b, 0x83, 0x49, 0x34, 0xf5, 0x3a, 0xa2, 0xe9, 0x10, 0x7a, 0x8b, 0x6c, 0x2,  0x2c};
	unsigned mac_len = 10;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_30_encrypt_ble_10mac_12) {
	printk("Testing Packet Vector #12\n");

	uint8_t K[16] = {0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x0D, 0x0C, 0x0B, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5};
	uint8_t A[] = {0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa, 0xb};
	uint8_t P[] = {0xc,  0xd,  0xe,  0xf,  0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16,
			0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f, 0x20};
	uint8_t C[] = {0xc0, 0xff, 0xa0, 0xd6, 0xf0, 0x5b, 0xdb, 0x67, 0xf2, 0x4d, 0x43, 0xa4, 0x33, 0x8d, 0x2a, 0xa4,
			0xbe, 0xd7, 0xb2, 0xe,  0x43, 0xcd, 0x1a, 0xa3, 0x16, 0x62, 0xe7, 0xad, 0x65, 0xd6, 0xdb};
	unsigned mac_len = 10;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_31_encrypt_ble_8mac_13) {
	printk("Testing Packet Vector #13\n");

	uint8_t K[16] = {0xD7, 0x82, 0x8D, 0x13, 0xB2, 0xB0, 0xBD, 0xC3, 0x25, 0xA7, 0x62, 0x36, 0xDF, 0x93, 0xCC, 0x6B};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x41, 0x2B, 0x4E, 0xA9, 0xCD, 0xBE, 0x3C, 0x96, 0x96, 0x76, 0x6C, 0xFA};
	uint8_t A[] = {0xb, 0xe1, 0xa8, 0x8b, 0xac, 0xe0, 0x18, 0xb1};
	uint8_t P[] = {0x8,  0xe8, 0xcf, 0x97, 0xd8, 0x20, 0xea, 0x25, 0x84, 0x60, 0xe9, 0x6a,
			0xd9, 0xcf, 0x52, 0x89, 0x5,  0x4d, 0x89, 0x5c, 0xea, 0xc4, 0x7c};
	uint8_t C[] = {0x4c, 0xb9, 0x7f, 0x86, 0xa2, 0xa4, 0x68, 0x9a, 0x87, 0x79, 0x47, 0xab, 0x80, 0x91, 0xef, 0x53,
			0x86, 0xa6, 0xff, 0xbd, 0xd0, 0x80, 0xf8, 0xe7, 0x8c, 0xf7, 0xcb, 0xc,  0xdd, 0xd7, 0xb3};
	unsigned mac_len = 8;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_32_encrypt_ble_8mac_14) {
	printk("Testing Packet Vector #14\n");

	uint8_t K[16] = {0xD7, 0x82, 0x8D, 0x13, 0xB2, 0xB0, 0xBD, 0xC3, 0x25, 0xA7, 0x62, 0x36, 0xDF, 0x93, 0xCC, 0x6B};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x33, 0x56, 0x8E, 0xF7, 0xB2, 0x63, 0x3C, 0x96, 0x96, 0x76, 0x6C, 0xFA};
	uint8_t A[] = {0x63, 0x1, 0x8f, 0x76, 0xdc, 0x8a, 0x1b, 0xcb};
	uint8_t P[] = {0x90, 0x20, 0xea, 0x6f, 0x91, 0xbd, 0xd8, 0x5a, 0xfa, 0x0,  0x39, 0xba,
			0x4b, 0xaf, 0xf9, 0xbf, 0xb7, 0x9c, 0x70, 0x28, 0x94, 0x9c, 0xd0, 0xec};
	uint8_t C[] = {0x4c, 0xcb, 0x1e, 0x7c, 0xa9, 0x81, 0xbe, 0xfa, 0xa0, 0x72, 0x6c, 0x55, 0xd3, 0x78, 0x6,  0x12,
			0x98, 0xc8, 0x5c, 0x92, 0x81, 0x4a, 0xbc, 0x33, 0xc5, 0x2e, 0xe8, 0x1d, 0x7d, 0x77, 0xc0, 0x8a};
	unsigned mac_len = 8;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_33_encrypt_ble_8mac_15) {
	printk("Testing Packet Vector #15\n");

	uint8_t K[16] = {0xD7, 0x82, 0x8D, 0x13, 0xB2, 0xB0, 0xBD, 0xC3, 0x25, 0xA7, 0x62, 0x36, 0xDF, 0x93, 0xCC, 0x6B};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x10, 0x3F, 0xE4, 0x13, 0x36, 0x71, 0x3C, 0x96, 0x96, 0x76, 0x6C, 0xFA};
	uint8_t A[] = {0xaa, 0x6c, 0xfa, 0x36, 0xca, 0xe8, 0x6b, 0x40};
	uint8_t P[] = {0xb9, 0x16, 0xe0, 0xea, 0xcc, 0x1c, 0x0,  0xd7, 0xdc, 0xec, 0x68, 0xec, 0xb,
			0x3b, 0xbb, 0x1a, 0x2,  0xde, 0x8a, 0x2d, 0x1a, 0xa3, 0x46, 0x13, 0x2e};
	uint8_t C[] = {0xb1, 0xd2, 0x3a, 0x22, 0x20, 0xdd, 0xc0, 0xac, 0x90, 0xd,  0x9a, 0xa0, 0x3c, 0x61, 0xfc, 0xf4, 0xa5,
			0x59, 0xa4, 0x41, 0x77, 0x67, 0x8,  0x97, 0x8,  0xa7, 0x76, 0x79, 0x6e, 0xdb, 0x72, 0x35, 0x6};
	unsigned mac_len = 8;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_34_encrypt_ble_8mac_16) {
	printk("Testing Packet Vector #16\n");

	uint8_t K[16] = {0xD7, 0x82, 0x8D, 0x13, 0xB2, 0xB0, 0xBD, 0xC3, 0x25, 0xA7, 0x62, 0x36, 0xDF, 0x93, 0xCC, 0x6B};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x76, 0x4C, 0x63, 0xB8, 0x05, 0x8E, 0x3C, 0x96, 0x96, 0x76, 0x6C, 0xFA};
	uint8_t A[] = {0xd0, 0xd0, 0x73, 0x5c, 0x53, 0x1e, 0x1b, 0xec, 0xf0, 0x49, 0xc2, 0x44};
	uint8_t P[] = {0x12, 0xda, 0xac, 0x56, 0x30, 0xef, 0xa5, 0x39, 0x6f, 0x77,
			0xc,  0xe1, 0xa6, 0x6b, 0x21, 0xf7, 0xb2, 0x10, 0x1c};
	uint8_t C[] = {0x14, 0xd2, 0x53, 0xc3, 0x96, 0x7b, 0x70, 0x60, 0x9b, 0x7c, 0xbb, 0x7c, 0x49, 0x91,
			0x60, 0x28, 0x32, 0x45, 0x26, 0x9a, 0x6f, 0x49, 0x97, 0x5b, 0xca, 0xde, 0xaf};
	unsigned mac_len = 8;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_35_encrypt_ble_8mac_17) {
	printk("Testing Packet Vector #17\n");

	uint8_t K[16] = {0xD7, 0x82, 0x8D, 0x13, 0xB2, 0xB0, 0xBD, 0xC3, 0x25, 0xA7, 0x62, 0x36, 0xDF, 0x93, 0xCC, 0x6B};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0xF8, 0xB6, 0x78, 0x09, 0x4E, 0x3B, 0x3C, 0x96, 0x96, 0x76, 0x6C, 0xFA};
	uint8_t A[] = {0x77, 0xb6, 0xf, 0x1, 0x1c, 0x3, 0xe1, 0x52, 0x58, 0x99, 0xbc, 0xae};
	uint8_t P[] = {0xe8, 0x8b, 0x6a, 0x46, 0xc7, 0x8d, 0x63, 0xe5, 0x2e, 0xb8,
			0xc5, 0x46, 0xef, 0xb5, 0xde, 0x6f, 0x75, 0xe9, 0xcc, 0xd};
	uint8_t C[] = {0x55, 0x45, 0xff, 0x1a, 0x8,  0x5e, 0xe2, 0xef, 0xbf, 0x52, 0xb2, 0xe0, 0x4b, 0xee,
			0x1e, 0x23, 0x36, 0xc7, 0x3e, 0x3f, 0x76, 0x2c, 0xc,  0x77, 0x44, 0xfe, 0x7e, 0x3c};
	unsigned mac_len = 8;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_36_encrypt_ble_8mac_18) {
	printk("Testing Packet Vector #18\n");

	uint8_t K[16] = {0xD7, 0x82, 0x8D, 0x13, 0xB2, 0xB0, 0xBD, 0xC3, 0x25, 0xA7, 0x62, 0x36, 0xDF, 0x93, 0xCC, 0x6B};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0xD5, 0x60, 0x91, 0x2D, 0x3F, 0x70, 0x3C, 0x96, 0x96, 0x76, 0x6C, 0xFA};
	uint8_t A[] = {0xcd, 0x90, 0x44, 0xd2, 0xb7, 0x1f, 0xdb, 0x81, 0x20, 0xea, 0x60, 0xc0};
	uint8_t P[] = {0x64, 0x35, 0xac, 0xba, 0xfb, 0x11, 0xa8, 0x2e, 0x2f, 0x7, 0x1d,
			0x7c, 0xa4, 0xa5, 0xeb, 0xd9, 0x3a, 0x80, 0x3b, 0xa8, 0x7f};
	uint8_t C[] = {0x0,  0x97, 0x69, 0xec, 0xab, 0xdf, 0x48, 0x62, 0x55, 0x94, 0xc5, 0x92, 0x51, 0xe6, 0x3,
			0x57, 0x22, 0x67, 0x5e, 0x4,  0xc8, 0x47, 0x9,  0x9e, 0x5a, 0xe0, 0x70, 0x45, 0x51};
	unsigned mac_len = 8;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_37_encrypt_ble_10mac_19) {
	printk("Testing Packet Vector #19\n");

	uint8_t K[16] = {0xD7, 0x82, 0x8D, 0x13, 0xB2, 0xB0, 0xBD, 0xC3, 0x25, 0xA7, 0x62, 0x36, 0xDF, 0x93, 0xCC, 0x6B};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x42, 0xFF, 0xF8, 0xF1, 0x95, 0x1C, 0x3C, 0x96, 0x96, 0x76, 0x6C, 0xFA};
	uint8_t A[] = {0xd8, 0x5b, 0xc7, 0xe6, 0x9f, 0x94, 0x4f, 0xb8};
	uint8_t P[] = {0x8a, 0x19, 0xb9, 0x50, 0xbc, 0xf7, 0x1a, 0x1,  0x8e, 0x5e, 0x67, 0x1,
			0xc9, 0x17, 0x87, 0x65, 0x98, 0x9,  0xd6, 0x7d, 0xbe, 0xdd, 0x18};
	uint8_t C[] = {0xbc, 0x21, 0x8d, 0xaa, 0x94, 0x74, 0x27, 0xb6, 0xdb, 0x38, 0x6a, 0x99, 0xac, 0x1a, 0xef, 0x23, 0xad,
			0xe0, 0xb5, 0x29, 0x39, 0xcb, 0x6a, 0x63, 0x7c, 0xf9, 0xbe, 0xc2, 0x40, 0x88, 0x97, 0xc6, 0xba};
	unsigned mac_len = 10;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_38_encrypt_ble_10mac_20) {
	printk("Testing Packet Vector #20\n");

	uint8_t K[16] = {0xD7, 0x82, 0x8D, 0x13, 0xB2, 0xB0, 0xBD, 0xC3, 0x25, 0xA7, 0x62, 0x36, 0xDF, 0x93, 0xCC, 0x6B};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x92, 0x0F, 0x40, 0xE5, 0x6C, 0xDC, 0x3C, 0x96, 0x96, 0x76, 0x6C, 0xFA};
	uint8_t A[] = {0x74, 0xa0, 0xeb, 0xc9, 0x6, 0x9f, 0x5b, 0x37};
	uint8_t P[] = {0x17, 0x61, 0x43, 0x3c, 0x37, 0xc5, 0xa3, 0x5f, 0xc1, 0xf3, 0x9f, 0x40,
			0x63, 0x2,  0xeb, 0x90, 0x7c, 0x61, 0x63, 0xbe, 0x38, 0xc9, 0x84, 0x37};
	uint8_t C[] = {0x58, 0x10, 0xe6, 0xfd, 0x25, 0x87, 0x40, 0x22, 0xe8, 0x3,  0x61, 0xa4,
			0x78, 0xe3, 0xe9, 0xcf, 0x48, 0x4a, 0xb0, 0x4f, 0x44, 0x7e, 0xff, 0xf6,
			0xf0, 0xa4, 0x77, 0xcc, 0x2f, 0xc9, 0xbf, 0x54, 0x89, 0x44};
	unsigned mac_len = 10;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_39_encrypt_ble_10mac_21) {
	printk("Testing Packet Vector #21\n");

	uint8_t K[16] = {0xD7, 0x82, 0x8D, 0x13, 0xB2, 0xB0, 0xBD, 0xC3, 0x25, 0xA7, 0x62, 0x36, 0xDF, 0x93, 0xCC, 0x6B};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x27, 0xCA, 0x0C, 0x71, 0x20, 0xBC, 0x3C, 0x96, 0x96, 0x76, 0x6C, 0xFA};
	uint8_t A[] = {0x44, 0xa3, 0xaa, 0x3a, 0xae, 0x64, 0x75, 0xca};
	uint8_t P[] = {0xa4, 0x34, 0xa8, 0xe5, 0x85, 0x0,  0xc6, 0xe4, 0x15, 0x30, 0x53, 0x88, 0x62,
			0xd6, 0x86, 0xea, 0x9e, 0x81, 0x30, 0x1b, 0x5a, 0xe4, 0x22, 0x6b, 0xfa};
	uint8_t C[] = {0xf2, 0xbe, 0xed, 0x7b, 0xc5, 0x9,  0x8e, 0x83, 0xfe, 0xb5, 0xb3, 0x16,
			0x8,  0xf8, 0xe2, 0x9c, 0x38, 0x81, 0x9a, 0x89, 0xc8, 0xe7, 0x76, 0xf1,
			0x54, 0x4d, 0x41, 0x51, 0xa4, 0xed, 0x3a, 0x8b, 0x87, 0xb9, 0xce};
	unsigned mac_len = 10;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_40_encrypt_ble_10mac_22) {
	printk("Testing Packet Vector #22\n");

	uint8_t K[16] = {0xD7, 0x82, 0x8D, 0x13, 0xB2, 0xB0, 0xBD, 0xC3, 0x25, 0xA7, 0x62, 0x36, 0xDF, 0x93, 0xCC, 0x6B};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x5B, 0x8C, 0xCB, 0xCD, 0x9A, 0xF8, 0x3C, 0x96, 0x96, 0x76, 0x6C, 0xFA};
	uint8_t A[] = {0xec, 0x46, 0xbb, 0x63, 0xb0, 0x25, 0x20, 0xc3, 0x3c, 0x49, 0xfd, 0x70};
	uint8_t P[] = {0xb9, 0x6b, 0x49, 0xe2, 0x1d, 0x62, 0x17, 0x41, 0x63, 0x28,
			0x75, 0xdb, 0x7f, 0x6c, 0x92, 0x43, 0xd2, 0xd7, 0xc2};
	uint8_t C[] = {0x31, 0xd7, 0x50, 0xa0, 0x9d, 0xa3, 0xed, 0x7f, 0xdd, 0xd4, 0x9a, 0x20, 0x32, 0xaa, 0xbf,
			0x17, 0xec, 0x8e, 0xbf, 0x7d, 0x22, 0xc8, 0x8,  0x8c, 0x66, 0x6b, 0xe5, 0xc1, 0x97};
	unsigned mac_len = 10;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_41_encrypt_ble_10mac_23) {
	printk("Testing Packet Vector #23\n");

	uint8_t K[16] = {0xD7, 0x82, 0x8D, 0x13, 0xB2, 0xB0, 0xBD, 0xC3, 0x25, 0xA7, 0x62, 0x36, 0xDF, 0x93, 0xCC, 0x6B};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x3E, 0xBE, 0x94, 0x04, 0x4B, 0x9A, 0x3C, 0x96, 0x96, 0x76, 0x6C, 0xFA};
	uint8_t A[] = {0x47, 0xa6, 0x5a, 0xc7, 0x8b, 0x3d, 0x59, 0x42, 0x27, 0xe8, 0x5e, 0x71};
	uint8_t P[] = {0xe2, 0xfc, 0xfb, 0xb8, 0x80, 0x44, 0x2c, 0x73, 0x1b, 0xf9,
			0x51, 0x67, 0xc8, 0xff, 0xd7, 0x89, 0x5e, 0x33, 0x70, 0x76};
	uint8_t C[] = {0xe8, 0x82, 0xf1, 0xdb, 0xd3, 0x8c, 0xe3, 0xed, 0xa7, 0xc2, 0x3f, 0x4,  0xdd, 0x65, 0x7,
			0x1e, 0xb4, 0x13, 0x42, 0xac, 0xdf, 0x7e, 0x0,  0xdc, 0xce, 0xc7, 0xae, 0x52, 0x98, 0x7d};
	unsigned mac_len = 10;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);

}

ZTEST(nrf_ccm_tests, test_ccm_42_encrypt_ble_10mac_24) {
	printk("Testing Packet Vector #24\n");

	uint8_t K[16] = {0xD7, 0x82, 0x8D, 0x13, 0xB2, 0xB0, 0xBD, 0xC3, 0x25, 0xA7, 0x62, 0x36, 0xDF, 0x93, 0xCC, 0x6B};
	uint8_t N[16] = {0x00, 0x00, 0x00, 0x00, 0x8D, 0x49, 0x3B, 0x30, 0xAE, 0x8B, 0x3C, 0x96, 0x96, 0x76, 0x6C, 0xFA};
	uint8_t A[] = {0x6e, 0x37, 0xa6, 0xef, 0x54, 0x6d, 0x95, 0x5d, 0x34, 0xab, 0x60, 0x59};
	uint8_t P[] = {0xab, 0xf2, 0x1c, 0xb,  0x2,  0xfe, 0xb8, 0x8f, 0x85, 0x6d, 0xf4,
			0xa3, 0x73, 0x81, 0xbc, 0xe3, 0xcc, 0x12, 0x85, 0x17, 0xd4};
	uint8_t C[] = {0xf3, 0x29, 0x5,  0xb8, 0x8a, 0x64, 0x1b, 0x4,  0xb9, 0xc9, 0xff, 0xb5, 0x8c, 0xc3, 0x90, 0x90,
			0xf,  0x3d, 0xa1, 0x2a, 0xb1, 0x6d, 0xce, 0x9e, 0x82, 0xef, 0xa1, 0x6d, 0xa6, 0x20, 0x59};
	unsigned mac_len = 10;
	uint8_t adata_mask = 0xFF;
	build_ccm_test_case_and_run(K, N, P, sizeof(P), sizeof(P), A, sizeof(A), C, sizeof(C), mac_len, adata_mask, false,
			false, ENCRYPT, BLE_MODE, NULL, false);
}

ZTEST_SUITE(nrf_ccm_tests, NULL, NULL, NULL, NULL, NULL);
